/* -*- c-basic-offset: 4 -*- */

#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "net.h"
#include "game.h"

pieceinfo_t pieceinfo[] = {
    { // NONE
	.num = 0,
	.name = "" },
    { 
	.num = 1,
        .name = "pharao" },
    { // DJHED
	.num = 2,
        .name = "djhed" },
    { // PYRAMID
	.num = 7,
	.name = "pyramid" },
    { // OBELISK
	.num = 4,
	.name = "obelisk" },
    { // OBELISK2
	.num = 0,
	.name = "stacked obelisk" },
};

sideinfo_t sideinfo[] = {
    { .ch = '.', .name = "silver" },
    { .ch = '=', .name = "red" }
};

char classic_setup[] =
    "H40" // PHARAO
    "E40E51" // DJHED
    "H22G73E22D21E91D92C32" // PYRAMID
    "H30H30H50H50" // OBELISK
    ;

int
game_turn(game_t *game)
{
    return game->move % 2;
}

int
laser_at(game_t *game, int x, int y)
{
    return game->laser_pos[0] == x && game->laser_pos[1] == y;
}

int
laser(game_t *game, int side) {
#define lreturn(v) do { retval=(v); goto ret; } while (0);
    int x, y, dir;
    int ddir;
    int retval;

    /*
      dir:
        0
      3 X 1
        2
     */

    //printf("LASER (%s)\n", sideinfo[side].name);

    assert(side == 0 || side == 1);
    switch (side) {
    case 0:
	x=9; y=7; dir=0;
	break;
    case 1:
	x=0; y=0; dir=2;
	break;
    }
  
    game->last_hit.piece = NONE;

    for (;;) {
	//printf("pos: (%d,%d)\n", y, x);
	switch (game->board[x][y].piece) {
	case NONE:
	    break;
	case PHARAO:
            game->last_hit = game->board[x][y];
	    game->winner = 1-game->board[x][y].side;
	    lreturn(SILVER_WINS + 1-game->board[x][y].side); /* GAME OVER */
	case OBELISK2:
            game->last_hit = game->board[x][y];
	    game->board[x][y].piece = OBELISK;
	    lreturn(L_PIECE);
	case OBELISK:
            game->last_hit = game->board[x][y];
	    game->board[x][y].piece = NONE;
	    lreturn(L_PIECE);
	case DJHED:
	    ddir = (game->board[x][y].dir - dir) & 3;
	    dir=(dir + 1 - 2*(ddir & 1)) & 3;
	    break;
	case PYRAMID:
	    ddir = (game->board[x][y].dir - dir) & 3;
	    if (ddir & 2) {
		game->last_hit = game->board[x][y];
		game->board[x][y].piece = NONE;
		lreturn(L_PIECE);
	    }
	    dir=(dir + 1 - 2*(ddir & 1)) & 3;
	    break;
	}
	switch (dir & 3) {
	case 0: y--;
	    if (y<0) lreturn(L_WALL);
	    break;
	case 1: x++;
	    if (x>BOARD_WIDTH-1) lreturn(L_WALL);
	    break;
	case 2: y++;
	    if (y>BOARD_HEIGHT-1) lreturn(L_WALL);
	    break;
	case 3: x--;
	    if (x<0) lreturn(L_WALL);
	    break;
	}
    }

ret:
    game->laser_pos[0] = x;
    game->laser_pos[1] = y;
    return retval;
}

int
square_allowed(int x, int y, int side)
{
    if (x == 0 && side == SILVER) return 0;
    if (x == BOARD_WIDTH-1 && side == RED) return 0;
    if (y == 0 || y == BOARD_HEIGHT-1) {
	if (x == 1 && side == RED) return 0;
	if (x == BOARD_WIDTH-2 && side == SILVER) return 0;
    }
    return 1;
}

int
movable(game_t *game, int x, int y, int side)
{
    return
	y>=0 && y<BOARD_HEIGHT &&
	x>=0 && x<BOARD_WIDTH &&
	game->board[x][y].piece != NONE &&
	game->board[x][y].side == side;
}

int
rotate(game_t *game, int side, int x, int y, rot_t dir)
{
    int incr;
    
    if (game->move % 2 != side)
	return E_NOT_YOUR_TURN;
    
    if (!movable(game, x, y, side))
	return E_ILLEGAL_MOVE;

    assert(dir==R_CW || dir==R_CCW);

    incr = dir==R_CW ? 1 : -1;
    game->board[x][y].dir = (game->board[x][y].dir + incr) & 3;
    game->move++;
    return HUGE_SUCCESS;
}

void
move_dest(move_t m, int *nx, int *ny)
{
    *ny = m.y - (m.dir<3) + ((m.dir>3) && (m.dir<7));
    *nx = m.x + ((m.dir>1) && (m.dir<5)) - ((m.dir<1) || (m.dir>5));
}

int
move(game_t *game, int side, move_t m)
{
    int x, y, dir, nx, ny;
    x = m.x;
    y = m.y;
    dir = m.dir;

    if (game->move % 2 != side)
	return E_NOT_YOUR_TURN;

    if (!movable(game, x, y, side))
	return E_ILLEGAL_MOVE;

    /* dir =
       0 1 2
       7 X 3
       6 5 4
    */

    assert((dir>=0) || (dir<=7));
    move_dest(m, &nx, &ny);
    if ((nx<0) || (nx>9) || (ny<0) || (ny>7))
	return E_ILLEGAL_MOVE;
  
    if (m.type == M_MOVE_SPLIT) {
	if (game->board[nx][ny].piece || (game->board[x][y].piece != OBELISK2))
	    return E_ILLEGAL_MOVE;
	game->board[x][y].piece = OBELISK;
	game->board[nx][ny].piece = OBELISK;
	game->board[nx][ny].side = game->board[x][y].side;
	return HUGE_SUCCESS;
    }
  
    if (!square_allowed(nx, ny, side))
	return E_ILLEGAL_MOVE;

    if (game->board[nx][ny].piece &&
	!((game->board[x][y].piece == DJHED) &&
	  ((game->board[nx][ny].piece != DJHED) &&
	   (game->board[nx][ny].piece != PHARAO))))
	return E_ILLEGAL_MOVE;

    struct square tmp;
    tmp = game->board[nx][ny];
    game->board[nx][ny] = game->board[x][y];
    game->board[x][y] = tmp;

    game->move++;
  
    return HUGE_SUCCESS;
}

void
place(game_t *game, int x, int y, int piece, int side, int dir)
{
    if (game->board[x][y].piece == OBELISK &&
	game->board[x][y].side == side &&
	piece == OBELISK) {
	piece = OBELISK2;
    } else if (game->board[x][y].piece != NONE &&
	       piece != NONE) {
	fprintf(stderr, "error: trying to place a piece on a non-empty square\n");
	exit(EXIT_FAILURE);
    }

    game->board[x][y].piece = piece;
    game->board[x][y].side = side;
    game->board[x][y].dir = dir;
}

int
perform_move(game_t *game, int side, move_t m)
{
    int retval;

    switch (m.type) {
    case M_MOVE:
    case M_MOVE_SPLIT:
	retval = move(game, side, m);
	break;
    case M_ROTATE:
	retval = rotate(game, side, m.x, m.y, m.dir);
	break;
    default:
	assert(0);
	return 0;
    }

    if (retval == HUGE_SUCCESS) {
	laser(game, side);
	game->last_move = m;
    }

    return retval;
}

int
parse_move(char *buf, move_t *m)
{
    if (strlen(buf) < 6)
	return E_FAILURE;

    switch (buf[0]) {
    case 'M': m->type = M_MOVE; break;
    case 'R': m->type = M_ROTATE; break;
    default: return E_FAILURE;
    }

    m->y = buf[2]-'A';
    m->x = buf[3]-'0';

    if (m->type == M_MOVE) {
	switch (buf[5]) {
	case 'N':
	    m->dir = (buf[6]=='W' ? D_NW : (buf[6]=='E' ? D_NE : D_N));
	    break;
	case 'S':
	    m->dir = (buf[6]=='W' ? D_SW : (buf[6]=='E' ? D_SE : D_S));
	    break;
	case 'E':
	    m->dir = D_E;
	    break;
	case 'W':
	    m->dir = D_W;
	    break;
	default:
	    return E_FAILURE;
	}
    } else if (m->type == M_ROTATE) {
	m->dir = (buf[5]=='L' ? R_CCW : R_CW);
    }

    return HUGE_SUCCESS;
}

void
clear_board(game_t *game)
{
    int x, y;
    for (y = 0; y < BOARD_HEIGHT; y++)
	for (x = 0; x < BOARD_WIDTH; x++)
	    place(game, x, y, 0, 0, 0);
}

void
setup_board_place_piece(game_t *game, int piece, int side, char *place_desc)
{
    int x, y, dir;

    y = place_desc[0]-'A';
    x = place_desc[1]-'0';
    dir = place_desc[2]-'0';

    if (side == 1) {
	y = BOARD_HEIGHT-y-1;
	x = BOARD_WIDTH-x-1;
	dir = (dir+2)&3;
    }

    place(game, x, y, piece, side, dir);
}

void
setup_board(game_t *game, char *desc)
{
    int side, piece, num, i;

    clear_board(game);

    for (side = 0; side < 2; side++) {
	i = 0;
	for (piece = 0; piece < N_PIECE_TYPES; piece++) {
	    for (num = 0; num < pieceinfo[piece].num; num++) {
		setup_board_place_piece(game, piece, side,
					&desc[i*3]);
		i++;
	    }
	}
    }
}

void
init_game(game_t *game)
{
    game->move = 0;
    game->winner = -1;
    game->laser_pos[0] = game->laser_pos[1] = -1;
    clear_board(game);
}

